package flm.b4a.scrollview2d;

import android.content.Context;
import anywheresoftware.b4a.BA;
import anywheresoftware.b4a.keywords.Common.DesignerCustomView;
import anywheresoftware.b4a.objects.CustomViewWrapper;
import anywheresoftware.b4a.objects.LabelWrapper;
import anywheresoftware.b4a.objects.PanelWrapper;
import anywheresoftware.b4a.objects.collections.Map;

@BA.ActivityObject
@BA.Version(1.3F)
@BA.ShortName("ScrollView2D")
@BA.Author("Frdric Leneuf-Magaud/Informatix")
@BA.Events(values = { "ScrollChanged(PosX As Int, PosY As Int)" })
@BA.DesignerProperties(values = {
		@BA.Property(key="FadingEdges", displayName="Fading edges", defaultValue="True", fieldType="Boolean"),
		@BA.Property(key="HorzScrollbarVisibility", displayName="Horizontal scrollbar", defaultValue="True", fieldType="Boolean"),
		@BA.Property(key="VertScrollbarVisibility", displayName="Vertical scrollbar", defaultValue="True", fieldType="Boolean"),
		@BA.Property(key="PanelHeight", displayName="Panel height", defaultValue="-1", fieldType="Int"),
		@BA.Property(key="PanelWidth", displayName="Panel width", defaultValue="-1", fieldType="Int")
})
public class ScrollView2DWrapper extends ViewWrapper<TwoDScrollView> implements DesignerCustomView
{
	/**
	 *ScrollView2D = HorizontalScrollView + VerticalScrollView
	 */
	public static void LIBRARY_DOC() {
		//
	}

	/**
	 *Initializes the ScrollView and sets its inner panel dimensions to the given width and height.
	 *You can later change these dimensions by calling ScrollView2D.Panel.Width or ScrollView2D.Panel.Height. 
	 *
	 *You should never use a ScrollView2D with a ListView, since ListView takes care of its own scrolling.
	 *Most importantly, doing this defeats all of the important optimizations in ListView for dealing with
	 *large lists, since it effectively forces the ListView to display its entire list of items to fill up
	 *the infinite container supplied by ScrollView2D.
	 */
	public void Initialize(BA ba, int Width, int Height, String EventName)
	{
		super.Initialize(ba, EventName);
		getObject().pw = new PanelWrapper();
		getObject().pw.Initialize(ba, "");
		getObject().addView(getObject().pw.getObject(), Width, Height);
	}

	@BA.Hide
	public void innerInitialize(BA ba, String EventName, boolean keepOldObject) {
		if (!keepOldObject)
			setObject(new MyScrollView(ba.context));
		super.innerInitialize(ba, EventName, true);
		if (getObject() instanceof MyScrollView) {
			if (ba.subExists(EventName + "_scrollchanged")) {
				MyScrollView m = (MyScrollView)getObject();
				m.ba = ba;
				m.eventName = EventName;
			}
		}
	}

	public PanelWrapper getPanel()
	{
		return getObject().pw;
	}

	final public static byte FS_DIR_HORZ = 0;
	final public static byte FS_DIR_VERT = 1;
	final public static byte FS_DIR_HORZ_AND_VERT = 2;
	final public static boolean FS_ToTheEND = true;

	/**
	 *Scrolls the scroll view to the beginning or end, horizontally or/and vertically.
	 *Direction:
	 *0 = horizontally 
	 *1 = vertically
	 *2 = horizontally and vertically
	 */
	public void FullScroll(byte Direction, boolean ToTheEnd, boolean Smoothly)
	{
		int NewX = getHorizontalScrollPosition();
		int NewY = getVerticalScrollPosition();
		if (ToTheEnd) {
			if (Direction > 0) {	// Vertically
				NewY = getPanel().getHeight();
			}
			if (Direction != 1) {	// Horizontally
				NewX = getPanel().getWidth();
			}
		}
		else {
			if (Direction > 0) {	// Vertically
				NewY = 0;
			}
			if (Direction != 1) {	// Horizontally
				NewX = 0;
			}
		}
		if (Smoothly) {
			SmoothScrollTo(NewX, NewY);
		}
		else {
			getObject().scrollTo(NewX, NewY);
		}
	}

	/**
	 *Sets the scrollbars visibility.
	 *They are visible by default.
	 */
	public void ScrollbarsVisibility(boolean HorizontalVisible, boolean VerticalVisible) {
		getObject().setHorizontalScrollBarEnabled(HorizontalVisible);
		getObject().setVerticalScrollBarEnabled(VerticalVisible);
	}

	/**
	 *Defines whether the edges should be faded when this view is scrolled.
	 *They are faded by default.
	 */
	public void FadingEdges(boolean Enabled) {
		getObject().setHorizontalFadingEdgeEnabled(Enabled);
		getObject().setVerticalFadingEdgeEnabled(Enabled);
	}

	/**
	 *Gets/sets the vertical scroll position.
	 */
	public int getVerticalScrollPosition() {
		return getObject().getScrollY();
	}

	public void setVerticalScrollPosition(int Scroll) {
		getObject().scrollTo(getHorizontalScrollPosition(), Scroll);
	}

	/**
	 *Gets/sets the horizontal scroll position.
	 */
	public int getHorizontalScrollPosition() {
		return getObject().getScrollX();
	}

	public void setHorizontalScrollPosition(int Scroll) {
		getObject().scrollTo(Scroll, getVerticalScrollPosition());
	}

	/**
	 *Scrolls smoothly to the given position.
	 */
	public void SmoothScrollTo(int PosX, int PosY) {
		getObject().smoothScrollTo(PosX, PosY);
	}

	/**
	 *Sets the focus to the first focusable view in the visible area of the ScrollView2D panel.
	 *You should call this function after a FullScroll, SmoothScrollTo or H/VScrollPosition to move the focus.
	 */
	public void GiveFocusToFirstVisible() {
		getObject().FindNewFocusedView(1, 1);
	}

	/**
	 *Prevents the ScrollView2D from intercepting the touch events.
	 *You should only call this function when the SV2D is in conflict with its parent about scrolling (e.g. AHViewPager).
	 */
	public void DisableTouchEventInterception(boolean Disabled) {
		getObject().DisallowInterceptTouchEvent = !Disabled;
	}

	/**
	 *Returns whether the ScrollView2D has finished scrolling.
	 */
	public boolean ScrollingIsFinished()
	{
		return getObject().ScrollingIsFinished();
	}

	@BA.Hide
	public static class MyScrollView extends TwoDScrollView {
		protected String eventName;
		protected BA ba;

		public MyScrollView(Context context) {
			super(context);
		}

		protected void onScrollChanged(int X, int Y, int oldX, int oldY)
		{
			super.onScrollChanged(X, Y, oldX, oldY);
			if (this.ba != null)
				this.ba.raiseEvent2(this, false, this.eventName + "_scrollchanged", true, new Object[] { X, Y });
		}
	}

	@Override
	public void DesignerCreateView(PanelWrapper base, LabelWrapper lw, Map props)
	{
		TwoDScrollView SV2D = getObject();
		SV2D.setBackgroundDrawable(base.getBackground());
		setEnabled(base.getEnabled());
		if (base.getTag().toString().length() == 0)
			setTag(null);
		else
			setTag(base.getTag());
		setVisible(base.getVisible());
		CustomViewWrapper.replaceBaseWithView(base, SV2D);

		FadingEdges((Boolean)props.Get("FadingEdges"));
		ScrollbarsVisibility((Boolean)props.Get("HorzScrollbarVisibility"), (Boolean)props.Get("VertScrollbarVisibility"));
		SV2D.pw.setHeight((int) props.Get("PanelHeight"));
		SV2D.pw.setWidth((int) props.Get("PanelWidth"));
	}

	@BA.Hide
	@Override
	public void _initialize(final BA ba, Object activityClass, String EventName)
	{
		Initialize(ba, -1, -1, EventName);
	}
}