package flm.b4a.scrollview2d;

import java.lang.reflect.Field;
import android.graphics.Bitmap;
import android.graphics.Rect;
import android.graphics.drawable.Drawable;
import android.graphics.drawable.GradientDrawable;
import android.view.View;
import android.view.ViewGroup;
import anywheresoftware.b4a.AbsObjectWrapper;
import anywheresoftware.b4a.BA;
import anywheresoftware.b4a.BALayout;
import anywheresoftware.b4a.BA.Hide;
import anywheresoftware.b4a.keywords.Common;
import anywheresoftware.b4a.objects.drawable.BitmapDrawable;
import anywheresoftware.b4a.objects.drawable.ColorDrawable;

@BA.Hide
public class ViewWrapper<T extends View> extends AbsObjectWrapper<T>
{
    protected BA ba;
    @Hide
    public static int lastId;

    public void Initialize(BA ba, String EventName)
    {
    	innerInitialize(ba, EventName.toLowerCase(BA.cul), false);
    }
    @Hide
    public void innerInitialize(final BA ba, final String eventName, boolean keepOldObject) {
    	this.ba = ba;
    	((View)getObject()).setId(++lastId);
    	if (ba.subExists(eventName + "_click"))
    	{
    		((View)getObject()).setOnClickListener(new View.OnClickListener()
    		{
    			public void onClick(View v)
    			{
    				ba.raiseEvent(getObject(), eventName + "_click", new Object[0]);
    			}
    		});
    	}

    	if (ba.subExists(eventName + "_longclick"))
    		((View)getObject()).setOnLongClickListener(new View.OnLongClickListener()
    		{
    			public boolean onLongClick(View v)
    			{
    				ba.raiseEvent(getObject(), eventName + "_longclick", new Object[0]);
    				return true;
    			}
    		});
    }
    /**
     * Gets or sets the background drawable.
     */
    public Drawable getBackground()
    {
    	return ((View)getObject()).getBackground();
    }
    public void setBackground(Drawable drawable)
    {
    	((View)getObject()).setBackgroundDrawable(drawable);
    }
    public void SetBackgroundImage(Bitmap Bitmap)
    {
    	BitmapDrawable bd = new BitmapDrawable();
    	bd.Initialize(Bitmap);
    	((View)getObject()).setBackgroundDrawable((Drawable)bd.getObject());
    }
    /**
     * Invalidates the whole view forcing the view to redraw itself.
     *Redrawing will only happen when the program can process messages. Usually when it finishes running the current code. 
     */
    public void Invalidate()
    {
    	((View)getObject()).invalidate();
    }
    /**
     * Invalidates the given rectangle.
     *Redrawing will only happen when the program can process messages. Usually when it finishes running the current code. 
     */
    public void Invalidate2(Rect Rect)
    {
    	getObject().invalidate(Rect);
    }
    /**
     * Invalidates the given rectangle.
     *Redrawing will only happen when the program can process messages. Usually when it finishes running the current code. 
     */
    public void Invalidate3(int Left, int Top, int Right, int Bottom)
    {
    	getObject().invalidate(Left, Top, Right, Bottom);
    }
    /**
     * Gets or sets the view's width.
     */
    public void setWidth(int width)
    {
    	ViewGroup.LayoutParams lp = ((View)getObject()).getLayoutParams();
    	lp.width = width;
    	((View)getObject()).getParent().requestLayout();
    }
    public int getWidth()
    {
    	return ((View)getObject()).getLayoutParams().width;
    }
    public int getHeight()
    {
    	return ((View)getObject()).getLayoutParams().height;
    }
    public int getLeft()
    {
    	BALayout.LayoutParams lp = (BALayout.LayoutParams)((View)getObject()).getLayoutParams();
    	return lp.left;
    }
    public int getTop()
    {
    	BALayout.LayoutParams lp = (BALayout.LayoutParams)((View)getObject()).getLayoutParams();
    	return lp.top;
    }
    /**
     * Gets or sets the view's height.
     */
    public void setHeight(int height)
    {
    	ViewGroup.LayoutParams lp = ((View)getObject()).getLayoutParams();
    	lp.height = height;
    	((View)getObject()).getParent().requestLayout();
    }
    /**
     * Gets or sets the view's left.
     */
    public void setLeft(int left)
    {
    	BALayout.LayoutParams lp = (BALayout.LayoutParams)((View)getObject()).getLayoutParams();
    	lp.left = left;
    	((View)getObject()).getParent().requestLayout();
    }
    /**
     * Gets or sets the view's top.
     */
    public void setTop(int top)
    {
    	BALayout.LayoutParams lp = (BALayout.LayoutParams)((View)getObject()).getLayoutParams();
    	lp.top = top;
    	((View)getObject()).getParent().requestLayout();
    }
    /**
     * Sets the background of the view to be a ColorDrawable with the given color.
     *If the current background is of type GradientDrawable or ColorDrawable the round corners will be kept.
     */
    public void setColor(int color)
    {
    	Drawable d = ((View)getObject()).getBackground();
    	if ((d != null) && ((d instanceof GradientDrawable))) {
    		float radius = 0.0F;
    		if ((d instanceof ColorDrawable.GradientDrawableWithCorners)) {
    			radius = ((ColorDrawable.GradientDrawableWithCorners)d).cornerRadius;
    		}
    		else {
    			GradientDrawable g = (GradientDrawable)((View)getObject()).getBackground();
    			try {
    				Field state = g.getClass().getDeclaredField("mGradientState");
    				state.setAccessible(true);
    				Object gstate = state.get(g);
    				Field radiusF = gstate.getClass().getDeclaredField("mRadius");
    				radius = ((Float)radiusF.get(gstate)).floatValue();
    			}
    			catch (Exception e) {
    				Common.Log(e.toString());
    			}
    		}
    		ColorDrawable cd = new ColorDrawable();
    		cd.Initialize(color, (int)radius);
    		((View)getObject()).setBackgroundDrawable((Drawable)cd.getObject());
    	}
    	else {
    		((View)getObject()).setBackgroundColor(color);
    	}
    }
    public void setTag(Object tag)
    {
    	((View)getObject()).setTag(tag);
    }
    /**
     * Gets or sets the Tag value. This is a place holder which can used to store additional data.
     */
    public Object getTag()
    {
    	return ((View)getObject()).getTag();
    }
    public void setVisible(boolean Visible)
    {
    	getObject().setVisibility(Visible ? View.VISIBLE : View.GONE);
    }
    public boolean getVisible()
    {
    	return getObject().getVisibility() == View.VISIBLE ? true : false;
    }
    public void setEnabled(boolean Enabled)
    {
    	getObject().setEnabled(Enabled);
    }
    public boolean getEnabled()
    {
    	return getObject().isEnabled();
    }
    /**
     * Changes the Z order of this view and brings it to the front.
     */
    public void BringToFront()
    {
    	((View)getObject()).bringToFront();
    }
    /**
     * Changes the Z order of this view and sends it to the back.
     */
    public void SendToBack()
    {
    	if ((((View)getObject()).getParent() instanceof ViewGroup)) {
    		ViewGroup vg = (ViewGroup)((View)getObject()).getParent();
    		vg.removeView((View)getObject());
    		vg.addView((View)getObject(), 0);
    	}
    }
    /**
     * Removes this view from its parent.
     */
    public void RemoveView()
    {
    	if ((((View)getObject()).getParent() instanceof ViewGroup)) {
    		ViewGroup vg = (ViewGroup)((View)getObject()).getParent();
    		vg.removeView((View)getObject());
    	}
    }
    /**
     * Changes the view position and size.
     */
    public void SetLayout(int Left, int Top, int Width, int Height)
    {
    	BALayout.LayoutParams lp = (BALayout.LayoutParams)((View)getObject()).getLayoutParams();
    	lp.left = Left;
    	lp.top = Top;
    	lp.width = Width;
    	lp.height = Height;
    	((View)getObject()).getParent().requestLayout();
    }
    /**
     * Tries to set the focus to this view.
     *Returns True if the focus was set.
     */
    public boolean RequestFocus()
    {
    	return getObject().requestFocus();
    }
}